@@ -154,11 +154,11 @@ Tk_GetAtomName(
 	if (mustFree) {
 	    XFree(mustFree);
 	}
-	name = Tcl_GetHashKey(&dispPtr->nameTable, hPtr);
+	name = (const char *)Tcl_GetHashKey(&dispPtr->nameTable, hPtr);
 	hPtr = Tcl_CreateHashEntry(&dispPtr->atomTable, INT2PTR(atom), &isNew);
 	Tcl_SetHashValue(hPtr, name);
     }
-    return Tcl_GetHashValue(hPtr);
+    return (const char *)Tcl_GetHashValue(hPtr);
 }
 
 /*
@@ -200,7 +200,7 @@ AtomInit(
 	name = atomNameArray[atom - 1];
 	hPtr = Tcl_CreateHashEntry(&dispPtr->nameTable, name, &isNew);
 	Tcl_SetHashValue(hPtr, INT2PTR(atom));
-	name = Tcl_GetHashKey(&dispPtr->nameTable, hPtr);
+	name = (const char *)Tcl_GetHashKey(&dispPtr->nameTable, hPtr);
 	hPtr = Tcl_CreateHashEntry(&dispPtr->atomTable, INT2PTR(atom), &isNew);
 	Tcl_SetHashValue(hPtr, name);
     }
