@@ -571,6 +571,9 @@ DisplayWinItem(
     if (winItemPtr->tkwin == NULL) {
 	return;
     }
+
+    Tcl_Preserve(canvas);
+
     if (state == TK_STATE_NULL) {
 	state = Canvas(canvas)->canvas_state;
     }
@@ -585,6 +588,7 @@ DisplayWinItem(
 	} else {
 	    Tk_UnmaintainGeometry(winItemPtr->tkwin, canvasTkwin);
 	}
+	Tcl_Release(canvas);
 	return;
     }
     Tk_CanvasWindowCoords(canvas, (double) winItemPtr->header.x1,
@@ -606,6 +610,7 @@ DisplayWinItem(
 	} else {
 	    Tk_UnmaintainGeometry(winItemPtr->tkwin, canvasTkwin);
 	}
+	Tcl_Release(canvas);
 	return;
     }
 
@@ -620,11 +625,16 @@ DisplayWinItem(
 		|| (height != Tk_Height(winItemPtr->tkwin))) {
 	    Tk_MoveResizeWindow(winItemPtr->tkwin, x, y, width, height);
 	}
-	Tk_MapWindow(winItemPtr->tkwin);
+
+	if (winItemPtr->tkwin) {
+	    Tk_MapWindow(winItemPtr->tkwin);
+	}
+
     } else {
 	Tk_MaintainGeometry(winItemPtr->tkwin, canvasTkwin, x, y,
 		width, height);
     }
+    Tcl_Release(canvas);
 }
 
 /*
@@ -789,7 +799,7 @@ WinItemToPostscript(
     Tk_Window tkwin = winItemPtr->tkwin;
 
     if (prepass || winItemPtr->tkwin == NULL) {
-        return TCL_OK;
+	return TCL_OK;
     }
 
     width = Tk_Width(tkwin);
