@@ -12,10 +12,12 @@
 
 #include "tkInt.h"
 
-#ifdef __WIN32__
+#ifdef _WIN32
 #include "tkWinInt.h"
-#elif !(defined(__WIN32__) || defined(MAC_OSX_TK))
+#elif !defined(MAC_OSX_TK)
 #include "tkUnixInt.h"
+#else
+#include "tkMacOSXInt.h"
 #endif
 
 /*
@@ -24,7 +26,7 @@
  * grab state information: the current grab window, the current restrict
  * window, and whether the mouse is captured.
  *
- * The current grab window specifies the point in the Tk window heirarchy
+ * The current grab window specifies the point in the Tk window hierarchy
  * above which pointer events will not be reported. Any window within the
  * subtree below the grab window will continue to receive events as normal.
  * Events outside of the grab tree will be reported to the grab window.
@@ -134,17 +136,6 @@ typedef struct NewGrabWinEvent {
 
 #define GENERATED_GRAB_EVENT_MAGIC ((Bool) 0x147321ac)
 
-/*
- * Mask that selects any of the state bits corresponding to buttons, plus
- * masks that select individual buttons' bits:
- */
-
-#define ALL_BUTTONS \
-	(Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)
-static const unsigned int buttonStates[] = {
-    Button1Mask, Button2Mask, Button3Mask, Button4Mask, Button5Mask
-};
-
 /*
  * Forward declarations for functions declared later in this file:
  */
@@ -177,7 +168,6 @@ static void		ReleaseButtonGrab(TkDisplay *dispPtr);
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 int
 Tk_GrabObjCmd(
     ClientData clientData,	/* Main window associated with interpreter. */
@@ -234,7 +224,7 @@ Tk_GrabObjCmd(
 	    Tcl_WrongNumArgs(interp, 1, objv, "?-global? window");
 	    return TCL_ERROR;
 	}
-	tkwin = Tk_NameToWindow(interp, arg, clientData);
+	tkwin = Tk_NameToWindow(interp, arg, (Tk_Window)clientData);
 	if (tkwin == NULL) {
 	    return TCL_ERROR;
 	}
@@ -250,7 +240,7 @@ Tk_GrabObjCmd(
 	    Tcl_WrongNumArgs(interp, 1, objv, "?-global? window");
 	    return TCL_ERROR;
 	}
-	tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]), clientData);
+	tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]), (Tk_Window)clientData);
 	if (tkwin == NULL) {
 	    return TCL_ERROR;
 	}
@@ -276,7 +266,7 @@ Tk_GrabObjCmd(
 	}
 	if (objc == 3) {
 	    tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]),
-		    clientData);
+		    (Tk_Window)clientData);
 	    if (tkwin == NULL) {
 		return TCL_ERROR;
 	    }
@@ -305,7 +295,7 @@ Tk_GrabObjCmd(
 	    Tcl_WrongNumArgs(interp, 1, objv, "release window");
 	    return TCL_ERROR;
 	}
-	tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]), clientData);
+	tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]), (Tk_Window)clientData);
 	if (tkwin == NULL) {
 	    Tcl_ResetResult(interp);
 	} else {
@@ -322,7 +312,7 @@ Tk_GrabObjCmd(
 	if (objc == 3) {
 	    globalGrab = 0;
 	    tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]),
-		    clientData);
+		    (Tk_Window)clientData);
 	} else {
 	    globalGrab = 1;
 
@@ -338,7 +328,7 @@ Tk_GrabObjCmd(
 		return TCL_ERROR;
 	    }
 	    tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[3]),
-		    clientData);
+		    (Tk_Window)clientData);
 	}
 	if (tkwin == NULL) {
 	    return TCL_ERROR;
@@ -355,7 +345,7 @@ Tk_GrabObjCmd(
 	    return TCL_ERROR;
 	}
 	winPtr = (TkWindow *) Tk_NameToWindow(interp, Tcl_GetString(objv[2]),
-		clientData);
+		(Tk_Window)clientData);
 	if (winPtr == NULL) {
 	    return TCL_ERROR;
 	}
@@ -426,12 +416,7 @@ Tk_Grab(
     }
 
     Tk_MakeWindowExist(tkwin);
-#ifndef MAC_OSX_TK
-    if (!grabGlobal)
-#else
-    if (0)
-#endif /* MAC_OSX_TK */
-    {
+    if (!grabGlobal) {
 	Window dummy1, dummy2;
 	int dummy3, dummy4, dummy5, dummy6;
 	unsigned int state;
@@ -654,7 +639,7 @@ Tk_Ungrab(
 
 static void
 ReleaseButtonGrab(
-    register TkDisplay *dispPtr)/* Display whose button grab is to be
+    TkDisplay *dispPtr)/* Display whose button grab is to be
 				 * released. */
 {
     unsigned int serial;
@@ -700,11 +685,11 @@ ReleaseButtonGrab(
 
 int
 TkPointerEvent(
-    register XEvent *eventPtr,	/* Pointer to the event. */
+    XEvent *eventPtr,	/* Pointer to the event. */
     TkWindow *winPtr)		/* Tk's information for window where event was
 				 * reported. */
 {
-    register TkWindow *winPtr2;
+    TkWindow *winPtr2;
     TkDisplay *dispPtr = winPtr->dispPtr;
     unsigned int serial;
     int outsideGrabTree = 0;
@@ -886,8 +871,9 @@ TkPointerEvent(
 		return 1;
 	    }
 	} else {
-	    if ((eventPtr->xbutton.state & ALL_BUTTONS)
-		    == buttonStates[eventPtr->xbutton.button - Button1]) {
+	    if (eventPtr->xbutton.button != AnyButton &&
+		    ((eventPtr->xbutton.state & ALL_BUTTONS)
+		    == TkGetButtonMask(eventPtr->xbutton.button))) {
 		ReleaseButtonGrab(dispPtr);			/* Note 4. */
 	    }
 	}
@@ -922,14 +908,14 @@ TkPointerEvent(
 
 void
 TkChangeEventWindow(
-    register XEvent *eventPtr,	/* Event to retarget. Must have type
+    XEvent *eventPtr,	/* Event to retarget. Must have type
 				 * ButtonPress, ButtonRelease, KeyPress,
 				 * KeyRelease, MotionNotify, EnterNotify, or
 				 * LeaveNotify. */
     TkWindow *winPtr)		/* New target window for event. */
 {
     int x, y, sameScreen, bd;
-    register TkWindow *childPtr;
+    TkWindow *childPtr;
 
     eventPtr->xmotion.window = Tk_WindowId(winPtr);
     if (eventPtr->xmotion.root ==
@@ -1010,7 +996,7 @@ TkInOutEvents(
     Tcl_QueuePosition position)	/* Position at which events are added to the
 				 * system event queue. */
 {
-    register TkWindow *winPtr;
+    TkWindow *winPtr;
     int upLevels, downLevels, i, j, focus;
 
     /*
@@ -1203,7 +1189,7 @@ MovePointer2(
 
 void
 TkGrabDeadWindow(
-    register TkWindow *winPtr)	/* Window that is in the process of being
+    TkWindow *winPtr)	/* Window that is in the process of being
 				 * deleted. */
 {
     TkDisplay *dispPtr = winPtr->dispPtr;
@@ -1293,7 +1279,7 @@ GrabRestrictProc(
     ClientData arg,
     XEvent *eventPtr)
 {
-    GrabInfo *info = arg;
+    GrabInfo *info = (GrabInfo *)arg;
     int mode, diff;
 
     /*
@@ -1351,7 +1337,7 @@ QueueGrabWindowChange(
 {
     NewGrabWinEvent *grabEvPtr;
 
-    grabEvPtr = ckalloc(sizeof(NewGrabWinEvent));
+    grabEvPtr = (NewGrabWinEvent *)ckalloc(sizeof(NewGrabWinEvent));
     grabEvPtr->header.proc = GrabWinEventProc;
     grabEvPtr->dispPtr = dispPtr;
     if (grabWinPtr == NULL) {
@@ -1386,7 +1372,7 @@ QueueGrabWindowChange(
 static int
 GrabWinEventProc(
     Tcl_Event *evPtr,		/* Event of type NewGrabWinEvent. */
-    int flags)			/* Flags argument to Tk_DoOneEvent: indicates
+    TCL_UNUSED(int))			/* Flags argument to Tcl_DoOneEvent: indicates
 				 * what kinds of events are being processed
 				 * right now. */
 {
@@ -1431,7 +1417,7 @@ FindCommonAncestor(
     int *countPtr2)		/* Store nesting level of winPtr2 within
 				 * common ancestor here. */
 {
-    register TkWindow *winPtr;
+    TkWindow *winPtr;
     TkWindow *ancestorPtr;
     int count1, count2, i;
 
