@@ -4,7 +4,7 @@
  *	This module implements the common elements of the Mac and Windows
  *	specific features of menus. This file is not used for UNIX.
  *
- * Copyright (c) 1996-1997 by Sun Microsystems, Inc.
+ * Copyright (c) 1996-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -13,7 +13,7 @@
 #include "tkInt.h"
 #include "tkMenu.h"
 
-typedef struct ThreadSpecificData {
+typedef struct {
     int postCommandGeneration;
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
@@ -43,10 +43,10 @@ PreprocessMenu(
     TkMenu *menuPtr)
 {
     int index, result, finished;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+    ThreadSpecificData *tsdPtr =
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    Tcl_Preserve((ClientData) menuPtr);
+    Tcl_Preserve(menuPtr);
 
     /*
      * First, let's process the post command on ourselves. If this command
@@ -67,7 +67,7 @@ PreprocessMenu(
     do {
 	finished = 1;
 	for (index = 0; index < menuPtr->numEntries; index++) {
-	    register TkMenuEntry *entryPtr = menuPtr->entries[index];
+	    TkMenuEntry *entryPtr = menuPtr->entries[index];
 
 	    if ((entryPtr->type == CASCADE_ENTRY)
 		    && (entryPtr->namePtr != NULL)
@@ -91,7 +91,7 @@ PreprocessMenu(
     } while (!finished);
 
   done:
-    Tcl_Release((ClientData) menuPtr);
+    Tcl_Release(menuPtr);
     return result;
 }
 
@@ -129,7 +129,7 @@ int
 TkPreprocessMenu(
     TkMenu *menuPtr)
 {
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+    ThreadSpecificData *tsdPtr =
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     tsdPtr->postCommandGeneration++;
