@@ -4,7 +4,7 @@
  *	Declarations shared among all of the files that implement menu
  *	widgets.
  *
- * Copyright (c) 1996-1998 by Sun Microsystems, Inc.
+ * Copyright (c) 1996-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -13,10 +13,6 @@
 #ifndef _TKMENU
 #define _TKMENU
 
-#ifndef _TK
-#include "tk.h"
-#endif
-
 #ifndef _TKINT
 #include "tkInt.h"
 #endif
@@ -75,8 +71,8 @@ typedef struct TkMenuEntry {
 				 * of character to underline (<0 means don't
 				 * underline anything). */
     Tcl_Obj *underlinePtr;	/* Index of character to underline. */
-    Tcl_Obj *bitmapPtr;		/* Bitmap to display in menu entry, or None.
-				 * If not None then label is ignored. */
+    Tcl_Obj *bitmapPtr;		/* Bitmap to display in menu entry, or NULL.
+				 * If not NULL then label is ignored. */
     Tcl_Obj *imagePtr;		/* Name of image to display, or NULL. If not
 				 * NULL, bitmap, text, and textVarName are
 				 * ignored. */
@@ -120,7 +116,7 @@ typedef struct TkMenuEntry {
 				 * always 0 for tearoff and separator
 				 * entries. */
     int hideMargin;		/* If this is 0, then the item has enough
-    				 * margin to accomodate a standard check mark
+    				 * margin to accommodate a standard check mark
     				 * and a default right margin. If this is 1,
     				 * then the item has no such margins, and
     				 * checkbuttons and radiobuttons with this set
@@ -175,7 +171,7 @@ typedef struct TkMenuEntry {
 				 * NULL means use overall disabledGC from menu
 				 * structure. See comments for disabledFg in
 				 * menu structure for more information. */
-    GC indicatorGC;		/* For drawing indicators. None means use GC
+    GC indicatorGC;		/* For drawing indicators. NULL means use GC
 				 * from menu. */
 
     /*
@@ -189,7 +185,7 @@ typedef struct TkMenuEntry {
     				 * the menu. */
 
     /*
-     * Bookeeping for master menus and cascade menus.
+     * Bookeeping for main menus and cascade menus.
      */
 
     struct TkMenuReferences *childMenuRefPtr;
@@ -270,7 +266,7 @@ typedef struct TkMenu {
     int numEntries;		/* Number of elements in entries. */
     int active;			/* Index of active entry. -1 means nothing
 				 * active. */
-    int menuType;		/* MASTER_MENU, TEAROFF_MENU, or MENUBAR. See
+    int menuType;		/* MAIN_MENU, TEAROFF_MENU, or MENUBAR. See
     				 * below for definitions. */
     Tcl_Obj *menuTypePtr;	/* Used to control whether created tkwin is a
 				 * toplevel or not. "normal", "menubar", or
@@ -340,7 +336,7 @@ typedef struct TkMenu {
     Tcl_Obj *takeFocusPtr;	/* Value of -takefocus option; not used in the
 				 * C code, but used by keyboard traversal
 				 * scripts. Malloc'ed, but may be NULL. */
-    Tcl_Obj *cursorPtr;		/* Current cursor for window, or None. */
+    Tcl_Obj *cursorPtr;		/* Current cursor for window, or NULL. */
     Tcl_Obj *postCommandPtr;	/* Used to detect cycles in cascade hierarchy
     				 * trees when preprocessing postcommands on
     				 * some platforms. See PostMenu for more
@@ -358,7 +354,7 @@ typedef struct TkMenu {
     struct TkMenu *masterMenuPtr;
     				/* A pointer to the original menu for this
     				 * clone chain. Points back to this structure
-    				 * if this menu is a master menu. */
+    				 * if this menu is a main menu. */
     void *reserved1; /* not used any more. */
     Tk_Window parentTopLevelPtr;/* If this menu is a menubar, this is the
     				 * toplevel that owns the menu. Only
@@ -437,7 +433,7 @@ typedef struct TkMenuReferences {
  * MENU_DELETION_PENDING	Non-zero means that we are currently
  *				destroying this menu's internal structures.
  *				This is useful when we are in the middle of
- *				cleaning this master menu's chain of menus up
+ *				cleaning this main menu's chain of menus up
  *				when TkDestroyMenu was called again on this
  *				menu (via a destroy binding or somesuch).
  * MENU_WIN_DESTRUCTION_PENDING Non-zero means we are in the middle of
@@ -455,15 +451,16 @@ typedef struct TkMenuReferences {
 #define MENU_PLATFORM_FLAG3	(1 << 28)
 
 /*
- * Each menu created by the user is a MASTER_MENU. When a menu is torn off, a
+ * Each menu created by the user is a MAIN_MENU. When a menu is torn off, a
  * TEAROFF_MENU instance is created. When a menu is assigned to a toplevel as
  * a menu bar, a MENUBAR instance is created. All instances have the same
- * configuration information. If the master instance is deleted, all instances
+ * configuration information. If the main instance is deleted, all instances
  * are deleted. If one of the other instances is deleted, only that instance
  * is deleted.
  */
 
 #define UNKNOWN_TYPE		-1
+#define MAIN_MENU 		0
 #define MASTER_MENU 		0
 #define TEAROFF_MENU 		1
 #define MENUBAR 		2
@@ -543,7 +540,9 @@ MODULE_SCOPE void	TkpMenuInit(void);
 MODULE_SCOPE int	TkpMenuNewEntry(TkMenuEntry *mePtr);
 MODULE_SCOPE int	TkpNewMenu(TkMenu *menuPtr);
 MODULE_SCOPE int	TkpPostMenu(Tcl_Interp *interp, TkMenu *menuPtr,
-			    int x, int y);
+			    int x, int y, int index);
+MODULE_SCOPE int	TkpPostTearoffMenu(Tcl_Interp *interp, TkMenu *menuPtr,
+			    int x, int y, int index);
 MODULE_SCOPE void	TkpSetWindowMenuBar(Tk_Window tkwin, TkMenu *menuPtr);
 
 #endif /* _TKMENU */
