@@ -411,10 +411,10 @@ TkTextMarkSegToIndex(
  *
  * Results:
  *	The return value is TCL_OK if "name" exists as a mark in the text
- *	widget and is located within its -starline/-endline range. In this
+ *	widget and is located within its -startline/-endline range. In this
  *	case *indexPtr is filled in with the next segment who is after the
  *	mark whose size is non-zero. TCL_ERROR is returned if the mark
- *	doesn't exist in the text widget, or if it is out of its -starline/
+ *	doesn't exist in the text widget, or if it is out of its -startline/
  *	-endline range. In this latter case *indexPtr still contains valid
  *	information, in particular TkTextMarkNameToIndex called with the
  *	"insert" or "current" mark name may return TCL_ERROR, but *indexPtr
@@ -434,8 +434,6 @@ TkTextMarkNameToIndex(
     TkTextIndex *indexPtr)	/* Index information gets stored here. */
 {
     TkTextSegment *segPtr;
-    TkTextIndex index;
-    int start, end;
 
     if (textPtr == NULL) {
         return TCL_ERROR;
@@ -456,28 +454,17 @@ TkTextMarkNameToIndex(
     }
     TkTextMarkSegToIndex(textPtr, segPtr, indexPtr);
 
-    /* If indexPtr refers to somewhere outside the -startline/-endline
+    /*
+     * If indexPtr refers to somewhere outside the -startline/-endline
      * range limits of the widget, error out since the mark indeed is not
      * reachable from this text widget (it may be reachable from a peer)
      * (bug 1630271).
      */
 
-    if (textPtr->start != NULL) {
-	start = TkBTreeLinesTo(NULL, textPtr->start);
-	TkTextMakeByteIndex(textPtr->sharedTextPtr->tree, NULL, start, 0,
-		&index);
-	if (TkTextIndexCmp(indexPtr, &index) < 0) {
-	    return TCL_ERROR;
-	}
-    }
-    if (textPtr->end != NULL) {
-	end = TkBTreeLinesTo(NULL, textPtr->end);
-	TkTextMakeByteIndex(textPtr->sharedTextPtr->tree, NULL, end, 0,
-		&index);
-	if (TkTextIndexCmp(indexPtr, &index) > 0) {
-	    return TCL_ERROR;
-	}
+    if (TkTextIndexAdjustToStartEnd(textPtr, indexPtr, 1) == TCL_ERROR) {
+	return TCL_ERROR;
     }
+
     return TCL_OK;
 }
 
