@@ -245,7 +245,7 @@ TkGetCursorByName(
     if ((argv[0][0] != '@') && !inTkTable) {
 	XColor fg, bg;
 	unsigned int maskIndex;
-	register const struct CursorName *namePtr;
+	const struct CursorName *namePtr;
 	TkDisplay *dispPtr;
 
 	/*
@@ -639,7 +639,6 @@ TkpFreeCursor(
     TkUnixCursor *unixCursorPtr = (TkUnixCursor *) cursorPtr;
 
     XFreeCursor(unixCursorPtr->display, (Cursor) unixCursorPtr->info.cursor);
-    Tk_FreeXId(unixCursorPtr->display, (XID) unixCursorPtr->info.cursor);
 }
 
 /*
