@@ -4,7 +4,7 @@
  *	This file provides functions that implement the "send" command,
  *	allowing commands to be passed from interpreter to interpreter.
  *
- * Copyright (c) 1997 by Sun Microsystems, Inc.
+ * Copyright (c) 1997 Sun Microsystems, Inc.
  * Copyright (c) 2003 Pat Thoyts <patthoyts@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
@@ -13,6 +13,7 @@
 
 #include "tkInt.h"
 #include "tkWinSendCom.h"
+#include "tkWinInt.h"
 
 /*
  * Should be defined in WTypes.h but mingw 1.0 is missing them.
@@ -121,6 +122,8 @@ Tk_SetAppName(
 				 * be globally unique. */
 {
 #ifndef TK_SEND_ENABLED_ON_WINDOWS
+    (void)tkwin;
+
     /*
      * Temporarily disabled for bug #858822
      */
@@ -150,7 +153,7 @@ Tk_SetAppName(
 	    return "";
 	}
 	tsdPtr->initialized = 1;
-	TRACE("Initialized COM library for interp 0x%08X\n", (long)interp);
+	TRACE("Initialized COM library for interp 0x%" TCL_Z_MODIFIER "x\n", (size_t)interp);
     }
 
     /*
@@ -163,7 +166,7 @@ Tk_SetAppName(
     if (riPtr == NULL) {
 	LPUNKNOWN *objPtr;
 
-	riPtr = ckalloc(sizeof(RegisteredInterp));
+	riPtr = (RegisteredInterp *)ckalloc(sizeof(RegisteredInterp));
 	memset(riPtr, 0, sizeof(RegisteredInterp));
 	riPtr->interp = interp;
 
@@ -213,6 +216,8 @@ TkGetInterpNames(
 				 * lookup. */
 {
 #ifndef TK_SEND_ENABLED_ON_WINDOWS
+    (void)interp;
+    (void)tkwin;
     /*
      * Temporarily disabled for bug #858822
      */
@@ -254,7 +259,8 @@ TkGetInterpNames(
 			    if (*p) {
 				Tcl_DString ds;
 
-				Tcl_WinTCharToUtf(p + 1, -1, &ds);
+				Tcl_DStringInit(&ds);
+				Tcl_WCharToUtfDString(p + 1, wcslen(p + 1), &ds);
 				result = Tcl_ListObjAppendElement(interp,
 					objList,
 					Tcl_NewStringObj(Tcl_DStringValue(&ds),
@@ -338,7 +344,7 @@ Tk_SendObjCmd(
      */
 
     for (i = 1; i < objc; i++) {
-	if (Tcl_GetIndexFromObjStruct(interp, objv[i], sendOptions,
+	if (Tcl_GetIndexFromObjStruct(NULL, objv[i], sendOptions,
 		sizeof(char *), "option", 0, &optind) != TCL_OK) {
 	    break;
 	}
@@ -619,7 +625,8 @@ BuildMoniker(
 	LPMONIKER pmkItem = NULL;
 	Tcl_DString dString;
 
-	Tcl_WinUtfToTChar(name, -1, &dString);
+	Tcl_DStringInit(&dString);
+	Tcl_UtfToWCharDString(name, -1, &dString);
 	hr = CreateFileMoniker((LPOLESTR)Tcl_DStringValue(&dString), &pmkItem);
 	Tcl_DStringFree(&dString);
 	if (SUCCEEDED(hr)) {
@@ -674,10 +681,10 @@ RegisterInterp(
 		    Tcl_DStringAppend(&dString, name, -1);
 		    Tcl_DStringAppend(&dString, " #", 2);
 		    offset = Tcl_DStringLength(&dString);
-		    Tcl_DStringSetLength(&dString, offset+TCL_INTEGER_SPACE);
+		    Tcl_DStringSetLength(&dString, offset + TCL_INTEGER_SPACE);
 		    actualName = Tcl_DStringValue(&dString);
 		}
-		sprintf(Tcl_DStringValue(&dString) + offset, "%d", i);
+		snprintf(Tcl_DStringValue(&dString) + offset, TCL_INTEGER_SPACE, "%d", i);
 	    }
 
 	    hr = BuildMoniker(actualName, &pmk);
@@ -733,8 +740,7 @@ Send(
 				 * object. */
     Tcl_Interp *interp,		/* The local interpreter. */
     int async,			/* Flag for the calling style. */
-    ClientData clientData,	/* The RegisteredInterp structure for this
-				 * interp. */
+    TCL_UNUSED(void *),
     int objc,			/* Number of arguments to be sent. */
     Tcl_Obj *const objv[])	/* The arguments to be sent. */
 {
@@ -761,8 +767,8 @@ Send(
 
     vCmd.vt = VT_BSTR;
     src = Tcl_GetString(cmd);
-    Tcl_WinUtfToTChar(src, cmd->length, &ds);
-    vCmd.bstrVal = SysAllocString((WCHAR *) Tcl_DStringValue(&ds));
+    Tcl_DStringInit(&ds);
+    vCmd.bstrVal = SysAllocString(Tcl_UtfToWCharDString(src, cmd->length, &ds));
     Tcl_DStringFree(&ds);
 
     dp.cArgs = 1;
@@ -784,8 +790,8 @@ Send(
 
     ehr = VariantChangeType(&vResult, &vResult, 0, VT_BSTR);
     if (SUCCEEDED(ehr)) {
-	Tcl_WinTCharToUtf(vResult.bstrVal, (int) SysStringLen(vResult.bstrVal) *
-		sizeof (WCHAR), &ds);
+	Tcl_DStringInit(&ds);
+	Tcl_WCharToUtfDString(vResult.bstrVal, SysStringLen(vResult.bstrVal), &ds);
 	Tcl_DStringResult(interp, &ds);
     }
 
@@ -797,8 +803,9 @@ Send(
 
     if (hr == DISP_E_EXCEPTION && ei.bstrSource != NULL) {
 	Tcl_Obj *opError, *opErrorCode, *opErrorInfo;
-	Tcl_WinTCharToUtf(ei.bstrSource, (int) SysStringLen(ei.bstrSource) *
-		sizeof (WCHAR), &ds);
+
+	Tcl_DStringInit(&ds);
+	Tcl_WCharToUtfDString(ei.bstrSource, SysStringLen(ei.bstrSource), &ds);
 	opError = Tcl_NewStringObj(Tcl_DStringValue(&ds),
 		Tcl_DStringLength(&ds));
 	Tcl_DStringFree(&ds);
@@ -869,14 +876,14 @@ TkWinSend_SetExcepInfo(
     /* TODO: Handle failure to append */
 
     src = Tcl_GetString(opError);
-    Tcl_WinUtfToTChar(src, opError->length, &ds);
+    Tcl_DStringInit(&ds);
     pExcepInfo->bstrDescription =
-	    SysAllocString((WCHAR *) Tcl_DStringValue(&ds));
+	    SysAllocString(Tcl_UtfToWCharDString(src, opError->length, &ds));
     Tcl_DStringFree(&ds);
     src = Tcl_GetString(opErrorCode);
-    Tcl_WinUtfToTChar(src, opErrorCode->length, &ds);
+    Tcl_DStringInit(&ds);
     pExcepInfo->bstrSource =
-	    SysAllocString((WCHAR *) Tcl_DStringValue(&ds));
+	    SysAllocString(Tcl_UtfToWCharDString(src, opErrorCode->length, &ds));
     Tcl_DStringFree(&ds);
     Tcl_DecrRefCount(opErrorCode);
     pExcepInfo->scode = E_FAIL;
@@ -923,7 +930,7 @@ TkWinSend_QueueCommand(
 
     TRACE("SendQueueCommand()\n");
 
-    evPtr = ckalloc(sizeof(SendEvent));
+    evPtr = (SendEvent *)ckalloc(sizeof(SendEvent));
     evPtr->header.proc = SendEventProc;
     evPtr->header.nextPtr = NULL;
     evPtr->interp = interp;
@@ -962,7 +969,7 @@ TkWinSend_QueueCommand(
 static int
 SendEventProc(
     Tcl_Event *eventPtr,
-    int flags)
+    TCL_UNUSED(int))
 {
     SendEvent *evPtr = (SendEvent *)eventPtr;
 
